@echo off

rem --------------------------------------------------------
rem 32bit vs 64 bit scripting engine

if "%ProgramFiles(x86)%" == "" goto thirtytwobit

set CScriptPath=%windir%\SYSWOW64\CSCRIPT.EXE

goto getparams

:thirtytwobit

set CScriptPath=cscript.exe

rem --------------------------------------------------------
rem Parse parameters


:getparams


if "%1" == "showstore" goto showstore
if "%1" == "cleanstore" goto cleanstore
if "%1" == "cleanfiles" goto cleanfiles
if "%1" == "create" goto create
if "%1" == "setupserver" goto setupserver
if "%1" == "setupclient" goto setupclient
if "%1" == "setupclientandserver" goto setupclientandserver

rem --------------------------------------------------------
rem Help

:usage

echo Simple WCF certificate management utility.
echo .
echo This script assumes the user operates a pair of certificates called
echo SimpleQueueServer and SimpleQueueClient.
echo If you want to have several pairs of client/server
echo (you will need them if you want to run several test/staging/production
echo deployments), you have to replace correspondent names in this script
echo and in your WCF configuration files (client and server).
echo .
echo Create command generates certificates that are valid
echo for 01/01/2000 - 01/01/2039 date range.
echo .
echo Deployed certificates are stored in "My" store at "LocalMachine" profile.
echo .
echo Usage:
echo .
echo cert showstore
echo cert cleanstore (WARNING: deletes ALL certificates from "localMachine My")
echo cert cleanfiles (WARNING: deletes ALL certificate files from current folder)
echo cert create     (WARNING: may overwrite existing certificate files)
echo cert setupserver serveraccount (e.g.: cert setupserver NetworkService)
echo cert setupclient clientaccount (e.g.: cert setupclient ASPNET)
echo cert setupclientandserver clientaccount serveraccount (useful for dev box setup)

goto end

: showstore

echo --------------------------------------------------------
echo Display store contents...
certmgr -c -s -r localMachine My

goto end

:cleanstore

echo --------------------------------------------------------
echo Delete all certificates from localMachine store...
certmgr -del -all -s -r LocalMachine My

goto end

:cleanfiles

del *.pfx
del *.pvk
del *.cer

goto end

:create

echo --------------------------------------------------------
echo Create client certificate (click None if you do not want to protect your private key with a password)...
makecert -r -pe -n "CN=SimpleQueueClient" -b 01/01/2000 -e 01/01/2039 -sky exchange SimpleQueueClient.cer -sv SimpleQueueClient.pvk

echo --------------------------------------------------------
echo Merge client public and privat keys on a pfx file ...
pvk2pfx.exe -pvk SimpleQueueClient.pvk -spc SimpleQueueClient.cer -pfx SimpleQueueClient.pfx    

echo --------------------------------------------------------
echo Create server certificate (click None if you do not want to protect your private key with a password)...
makecert -r -pe -n "CN=SimpleQueueServer" -b 01/01/2000 -e 01/01/2039 -sky exchange SimpleQueueServer.cer -sv SimpleQueueServer.pvk

echo --------------------------------------------------------
echo Merge server public and privat keys on a pfx file...
pvk2pfx.exe -pvk SimpleQueueServer.pvk -spc SimpleQueueServer.cer -pfx SimpleQueueServer.pfx    

echo --------------------------------------------------------
echo Delete private key files...
del *.pvk

goto end

:setupclient

if "%2" == "" goto usage

echo --------------------------------------------------------
echo Add client certificate (public and private keys) from exported file to store...
%CScriptPath% cstore.vbs import -l LM -s My SimpleQueueClient.pfx

echo --------------------------------------------------------
echo Make client key accessible for %2...
for /F "delims=" %%i in ('FindPrivateKey.exe My localMachine -n CN^=SimpleQueueClient -a') do set PRIVATE_KEY_FILE=%%i
echo Y|cacls.exe "%PRIVATE_KEY_FILE%" /E /G %2:R

echo --------------------------------------------------------
echo Add server certificate (public key only) from exported file to store...
certmgr -add SimpleQueueServer.cer -r localMachine -c -n SimpleQueueServer -s My 

echo --------------------------------------------------------
echo Display store contents...
certmgr -c -s -r localMachine My


goto end

:setupserver

if "%2" == "" goto usage

echo --------------------------------------------------------
echo Add server certificate (public and private keys) from exported file to store...
%CScriptPath% cstore.vbs import -l LM -s My SimpleQueueServer.pfx


echo --------------------------------------------------------
echo Make server key accessible for %2...
for /F "delims=" %%i in ('FindPrivateKey.exe My localMachine -n CN^=SimpleQueueServer -a') do set PRIVATE_KEY_FILE=%%i
echo Y|cacls.exe "%PRIVATE_KEY_FILE%" /E /G %2:R

echo --------------------------------------------------------
echo Add client certificate (public key only) from exported file to store...
certmgr -add SimpleQueueClient.cer -r localMachine -c -n SimpleQueueClient -s My 

goto end

:setupclientandserver

if "%2" == "" goto usage
if "%3" == "" goto usage

echo --------------------------------------------------------
echo Add client certificate (public and private keys) from exported file to store...
%CScriptPath% cstore.vbs import -l LM -s My SimpleQueueClient.pfx

echo --------------------------------------------------------
echo Make client key accessible for %2...
for /F "delims=" %%i in ('FindPrivateKey.exe My localMachine -n CN^=SimpleQueueClient -a') do set PRIVATE_KEY_FILE=%%i
echo Y|cacls.exe "%PRIVATE_KEY_FILE%" /E /G %2:R

echo --------------------------------------------------------
echo Add server certificate (public and private keys) from exported file to store...
%CScriptPath% cstore.vbs import -l LM -s My SimpleQueueServer.pfx

echo --------------------------------------------------------
echo Make server key accessible for %3...
for /F "delims=" %%i in ('FindPrivateKey.exe My localMachine -n CN^=SimpleQueueServer -a') do set PRIVATE_KEY_FILE=%%i
echo Y|cacls.exe "%PRIVATE_KEY_FILE%" /E /G %3:R

:end

rem ...............Samples.....................

rem echo --------------------------------------------------------
rem echo Save cert to TrustedPeople...
rem certmgr.exe -add -r localMachine -s My -c -n SimpleQueueServer -r LocalMachine -s TrustedPeople

rem echo --------------------------------------------------------
rem echo Export client certificate...
rem certmgr -put -r CurrentUser -s My -c -n SimpleClientQueue SimpleClientQueue.cer

rem Unfortunately, certmgr doesn't work properly with pfx, so we have to use capicom.dll (cstore.vbs)
rem certmgr -add SimpleQueueServer.pfx -r localMachine -c -n SimpleQueueServer -s My 
rem %CScriptPath% cstore.vbs import -l LM -s My SimpleQueueServer.pfx
